#!/usr/bin/env python
#--------------------------------------------------------------------------------------------------------------
# Copyright (c) International Business Machines Corp., 2008
#
# IBM XIV Storage System Software
#
# Licensed materials provided under the terms of the IBM International Program License Agreement.
# See the software licensing materials that came with the IBM XIV Storage System for terms and conditions.
#
#--------------------------------------------------------------------------------------------------------------


from host_attach import logger
from platform import system
from sys import exit
from host_attach import logger, is_error_free, execute_and_log, PACKAGE_DIR

DRIVERS_PATH = PACKAGE_DIR + '/drivers/linux/qlogic'
QLOGIC_DRIVER_PATH = DRIVERS_PATH + '/qlafc-linux-8.02.14.01-1-install.tgz'
QLOGIC_LUN_SCAN_SCRIPT = PACKAGE_DIR + '/bin/qlogic_lun_scan'
QLOGIC_MINIMAL_DRIVER_VERSION  = '8.02'

def qlogic_install():
        file_path_without_extension = DRIVERS_PATH + '/qlafc-linux-8.02.14.01-1-install'
        output = execute_and_log('tar -zxf ' + QLOGIC_DRIVER_PATH + ' -C ' + DRIVERS_PATH)
        if not is_error_free(output):
                logger.debug('failed to extract ' + QLOGIC_DRIVER_PATH)
                return 0
        output = execute_and_log('cd ' + file_path_without_extension + ' && ./qlinstall --force')
        if not is_error_free(output):
                logger.error('qlogic installation script returned an error, check for error in logfile /var/log/host_attach')
                execute_and_log('popd')
                return 0
        execute_and_log('popd')
        return 1

def qlogic_upgrade():
        file_path_without_extension = DRIVERS_PATH + '/qlafc-linux-8.02.14.01-1-install'
        output = execute_and_log('tar -zxf ' + QLOGIC_DRIVER_PATH + ' -C ' + DRIVERS_PATH)
        if not is_error_free(output):
                logger.debug('failed to extract ' + QLOGIC_DRIVER_PATH)
                return 0
        output = execute_and_log('cd ' + file_path_without_extension + ' && ./qlinstall --upgrade')
        if not is_error_free(output):
                logger.error('qlogic installation script returned an error, check for error in logfile /var/log/host_attach')
                execute_and_log('popd')
                return 0
        execute_and_log('popd')
        return 1

def is_qlogic_module(module_name):
	if module_name.__contains__('ql'):
		return 1
	if module_name.__contains__('QL'):
		return 1
	if module_name.__contains__('<NULL>'):
		return 1
	return 0

def qlogic_extract_host_from_module(module_name):
	logger.debug('extracting host_name from ' + module_name)
	elements = module_name.split('/')
	host_name = ''
	for element in elements:
		if element.__contains__('host') and not element.__contains__('scsi_host'):
			host_name = element
	logger.debug('host_name found is ' + host_name)
	return host_name

def qlogic_get_scsi_hosts():
	hosts = []
	all_proc_filenames = execute_and_log('find /sys/class/scsi_host/ -type f | grep proc_name')
	all_proc_filenames = all_proc_filenames.splitlines()
	for proc_filename in all_proc_filenames:
		module_name = execute_and_log('cat ' + proc_filename)
		if is_qlogic_module(module_name):
			host = qlogic_extract_host_from_module(proc_filename)
			model_name = execute_and_log('cat /sys/class/scsi_host/' + host + '/model_name')
			proc_name = execute_and_log('cat ' + proc_filename)
			logger.debug('model_name ' + model_name + ',,, proc_name ' + proc_name)
			if proc_name == '<NULL>' and not is_qlogic_module(model_name):
				logger.debug('not qlogic module')
			else:
				logger.debug('appending host ' + host)
				hosts.append(host)
	return hosts

def qlogic_get_driver_version_from_sysfs(host):
	output = execute_and_log('cat /sys/class/scsi_host/' + host + '/driver_version')
	return output

def qlogic_get_host_id_from_host_name(host_name):
	return host_name.split('host')[1]

def qlogic_get_driver_version_from_string(driver_string):
	elements = driver_string.split()
	count = 0
	for item in elements:
		count = count +1
	long_driver_version = elements[count-1]
	driver_version = long_driver_version.split('-')[0]
	return driver_version

def qlogic_get_driver_version_from_proc(host):
	host_id = qlogic_get_host_id_from_host_name(host)
	driver_string = execute_and_log('cat /proc/scsi/qla2xxx/' + host_id + ' | grep "Driver version"')
	driver_version = qlogic_get_driver_version_from_string(driver_string)
	return driver_version

def qlogic_get_driver_version():
	hosts = qlogic_get_scsi_hosts()
	host = hosts[0]
	driver_version = qlogic_get_driver_version_from_proc(host)
	if driver_version == '':
		driver_version = qlogic_get_driver_version_from_sysfs(host)
		if not is_error_free(driver_version):
			logger.debug('failed to query driver version')
			return '99999999'
	return driver_version

def is_qlogic_driver_installed():
	output = execute_and_log('ls /lib/modules/`uname -r`/kernel/drivers/scsi/qla2xxx')
	if not is_error_free(output):
		logger.debug('qlogic driver is not installed')
		return 0
	return 1
	
def is_qlogic_driver_loaded():
	output = execute_and_log('lsmod | grep qla2xxx')
	if not is_error_free(output):
		logger.debug('qlogic driver qla2xxx not loaded')
		return 0
	if output == '':
		return 0
	return 1

def is_qlogic_driver_up2date():
	driver_version = qlogic_get_driver_version()
	if driver_version < QLOGIC_MINIMAL_DRIVER_VERSION:
		logger.debug('the installed qlogic driver ' + driver_version + 'is older then required: ' + QLOGIC_MINIMAL_DRIVER_VERSION)
		return 0
	return 1

def qlogic_verify():
	if not is_qlogic_driver_installed():
		logger.error('qlogic driver is not installed, use --install')
		return 0
	if not is_qlogic_driver_loaded():
		logger.error('qlogic driver is not loaded, probably not installed')
		return 0
	if not is_qlogic_driver_up2date():
		logger.error('installed qlogic driver version does not meet minimal required version, use --upgrade')
		return 0
	logger.debug('the installed qlogic driver is recent enough')
	return 1

def qlogic_uninstall():
	file_path_without_extension = DRIVERS_PATH + '/qlafc-linux-8.02.14.01-1-install'
	output = execute_and_log('tar -zxf ' + QLOGIC_DRIVER_PATH + ' -C ' + DRIVERS_PATH)
	if not is_error_free(output):
		logger.debug('failed to extract ' + QLOGIC_DRIVER_PATH)
		return 0
	output = execute_and_log('cd ' + file_path_without_extension + ' && ./qlinstall --uninstall')
	if not is_error_free(output):
		logger.error('qlogic installation script returned an error, check for error in logfile /var/log/host_attach')
		execute_and_log('popd')
		return 0
        execute_and_log('popd')
	return 1

def qlogic_scan():
	output = execute_and_log(QLOGIC_LUN_SCAN_SCRIPT + ' --scan')
	if not is_error_free(output):
		logger.debug('failed to execute ' + QLOGIC_LUN_SCAN_SCRIPT + ' --scan')
		return 0
	return 1

def qlogic_get_wwn_from_sysfs(host):
	output = execute_and_log('cat /sys/class/scsi_host/' + host + '/device/fc_host\:' + host + '/port_name')
	return output

def qlogic_get_wwn_from_proc(host):
	host_id = qlogic_get_host_id_from_host_name(host)
	wwn_string = execute_and_log('cat /proc/scsi/qla2xxx/' + host_id + ' | grep adapter-port')
	return wwn_string
	
def qlogic_wwn_print():
	logger.info('WWN for existing devices:')
	hosts = qlogic_get_scsi_hosts()
	for host in hosts:
		logger.debug('querying wwn for host ' + host)
		wwn = qlogic_get_wwn_from_sysfs(host)
		if not is_error_free(wwn):
			wwn = qlogic_get_wwn_from_proc(host)
		logger.debug(wwn)
		print wwn
	return 1
